﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2013 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System.Collections.Generic;
using System.Windows;
using System.Windows.Media;
using System.Windows.Shapes;
using ICSharpCode.AvalonEdit.Document;
using ICSharpCode.AvalonEdit.Rendering;

namespace HeuristicLab.Problems.GPDL.Views {
  public class ColorizeErrors : DocumentColorizingTransformer {
    private Dictionary<int, string> synErrors = new Dictionary<int, string>();
    private Dictionary<int, string> lexErrors = new Dictionary<int, string>();

    protected override void ColorizeLine(DocumentLine line) {
      if (lexErrors.ContainsKey(line.LineNumber)) {
        ChangeLinePart(line.Offset, line.Offset + line.Length, ChangeLexError);
      } else if (synErrors.ContainsKey(line.LineNumber)) {
        ChangeLinePart(line.Offset, line.Offset + line.Length, ChangeSynError);
      }
      //int lineStartOffset = line.Offset;
      //string text = CurrentContext.Document.GetText(line);
      //int start = 0;
      //int index;
      //while ((index = text.IndexOf("AvalonEdit", start)) >= 0) {
      //  base.ChangeLinePart(
      //    lineStartOffset + index, // startOffset
      //    lineStartOffset + index + 10, // endOffset
      //    (VisualLineElement element) => {
      //      // This lambda gets called once for every VisualLineElement
      //      // between the specified offsets.
      //      Typeface tf = element.TextRunProperties.Typeface;
      //      // Replace the typeface with a modified version of
      //      // the same typeface
      //      element.TextRunProperties.SetTypeface(new Typeface(
      //        tf.FontFamily,
      //        FontStyles.Italic,
      //        FontWeights.Bold,
      //        tf.Stretch
      //      ));
      //    });
      //  start = index + 1; // search for next occurrence
      //}
    }

    private void ChangeSynError(VisualLineElement element) {


      var wigglePath = new Path();
      wigglePath.Data = Geometry.Parse("M 0,1 C 1,0 2,2 3,1");
      wigglePath.Stroke = Brushes.Red;
      wigglePath.StrokeThickness = 0.2;
      wigglePath.StrokeEndLineCap = PenLineCap.Square;
      wigglePath.StrokeStartLineCap = PenLineCap.Square;

      var wigglyBrush = new VisualBrush();
      wigglyBrush.Viewbox = new Rect(0, 0, 3, 2);
      wigglyBrush.ViewboxUnits = BrushMappingMode.Absolute;
      wigglyBrush.Viewport = new Rect(0, 0.8, 6, 4);
      wigglyBrush.ViewportUnits = BrushMappingMode.Absolute;
      wigglyBrush.TileMode = TileMode.Tile;
      wigglyBrush.Visual = wigglePath;


      var wigglyPen = new Pen(wigglyBrush, 6.0);
      var decorations = new TextDecorationCollection();
      decorations.Add(new TextDecoration(0, wigglyPen, 0, TextDecorationUnit.Pixel, TextDecorationUnit.Pixel));
      element.TextRunProperties.SetTextDecorations(decorations);
      //element.TextRunProperties.SetTypeface(new Typeface(
      //  tf.FontFamily,
      //  FontStyles.Italic,
      //  FontWeights.Bold,
      //  tf.Stretch
      //));
    }

    private void ChangeLexError(VisualLineElement element) {

      var wigglePath = new Path();
      wigglePath.Data = Geometry.Parse("M 0,1 C 1,0 2,2 3,1");
      wigglePath.Stroke = Brushes.Red;
      wigglePath.StrokeThickness = 0.2;
      wigglePath.StrokeEndLineCap = PenLineCap.Square;
      wigglePath.StrokeStartLineCap = PenLineCap.Square;

      var wigglyBrush = new VisualBrush();
      wigglyBrush.Viewbox = new Rect(0, 0, 3, 2);
      wigglyBrush.ViewboxUnits = BrushMappingMode.Absolute;
      wigglyBrush.Viewport = new Rect(0, 0.8, 6, 4);
      wigglyBrush.ViewportUnits = BrushMappingMode.Absolute;
      wigglyBrush.TileMode = TileMode.Tile;
      wigglyBrush.Visual = wigglePath;


      var wigglyPen = new Pen(wigglyBrush, 6.0);
      var decorations = new TextDecorationCollection();
      decorations.Add(new TextDecoration(0, wigglyPen, 0, TextDecorationUnit.Pixel, TextDecorationUnit.Pixel));
      element.TextRunProperties.SetTextDecorations(decorations);

      //element.TextRunProperties.SetTypeface(new Typeface(
      //  tf.FontFamily,
      //  FontStyles.Italic,
      //  FontWeights.Bold,
      //  tf.Stretch
      //));
    }

    internal void MarkSynError(string msg, int line, int col) {
      if (!synErrors.ContainsKey(line)) {
        synErrors.Add(line, msg);
      }
    }

    internal void MarkLexError(string msg, int line, int col) {
      if (!lexErrors.ContainsKey(line)) {
        lexErrors.Add(line, msg);
      }
    }

    internal void ClearErrors() {
      synErrors.Clear();
      lexErrors.Clear();
    }
  }
}
